@@ -39,6 +39,9 @@ from sqlalchemy.exc import (
 )
 
 
+_KEYSIZE = 1000-64
+_VALSIZE = 4000
+_NSSIZE  = 64
 
 class SQLDict(DictMixin):
     '''
@@ -46,9 +49,6 @@ class SQLDict(DictMixin):
     >>> d = SQLDict(e.connect(), 'asd', create=1)
     '''
 
-    _KEYSIZE = 1000-64
-    _VALSIZE = 4000
-    _NSSIZE  = 64
 
     _F_SERIALIZED_KEY = 1 << 0
     _F_SERIALIZED_VAL = 1 << 1
@@ -129,9 +129,9 @@ class SQLDict(DictMixin):
         valcol = self._valcol
         keytype = self._keytype
         valtype = self._valtype
-        keysize = self._KEYSIZE
-        valsize = self._VALSIZE
-        nssize = self._NSSIZE
+        keysize = self._keysize
+        valsize = self._valsize
+        nssize = self._nssize
         ns = self._ns
         extended = self._extended
         #
@@ -190,8 +190,9 @@ class SQLDict(DictMixin):
 
     def _pack_key(self, key, flags):
         extended = self._extended
+        keycoltype = self._keytype
         #
-        if extended:
+        if extended and isinstance(keycoltype, String):
             key_type = type(key)
             if key_type is str:
                 pass
@@ -204,8 +205,9 @@ class SQLDict(DictMixin):
 
     def _unpack_key(self, key, flags):
         extended = self._extended
+        keycoltype = self._keytype
         #
-        if extended:
+        if extended and isinstance(keycoltype, String):
             if flags & self._F_SERIALIZED_KEY:
                 key = self._deserialize(key)
             elif type(key) is unicode and not (flags & self._F_UNICODE_KEY):
@@ -215,8 +217,9 @@ class SQLDict(DictMixin):
 
     def _pack_val(self, val, flags):
         extended = self._extended
+        valcoltype = self._valtype
         #
-        if extended:
+        if extended and isinstance(valcoltype, String):
             val_type = type(val)
             if val_type is str:
                 pass
@@ -229,8 +232,9 @@ class SQLDict(DictMixin):
 
     def _unpack_val(self, val, flags):
         extended = self._extended
+        valcoltype = self._valtype
         #
-        if extended:
+        if extended and isinstance(valcoltype, String):
             if flags & self._F_SERIALIZED_VAL:
                 val = self._deserialize(val)
             elif type(val) is unicode and not (flags & self._F_UNICODE_VAL):
@@ -573,27 +577,38 @@ def sqldict(db, tablename,
                     serialize=0,
                     create=0,
                     sort=1,
-                    **kw):
+                    keycol="sqldict_key",
+                    valcol="sqldict_val",
+                    keytype=String(_KEYSIZE),
+                    valtype=String(_VALSIZE)
+                    ):
+
     """
     The various dictionaries, dN, can be used just as an ordinary dict.
 
     >>> #d0 = sqldict("mysql://user:pass@dbhost/dbname", "table0", create=1)
-    >>> engine = create_engine("sqlite://")
-    >>> d1 = sqldict(engine, "table2", create=1)
-    >>> d2 = sqldict(engine, "table3", create=2, valtype=Integer)
+    >>> e = create_engine("sqlite://")
+    >>> d1 = sqldict(e, "table2", create=1)
+    >>> d2 = sqldict(e, "table3", create=2, valtype=Integer)
     >>> contents = {"asd":123}
     >>> d1.update(contents)
     >>> d2.update(contents)
     >>> assert d1["asd"] == "123"
     >>> assert d2["asd"] == 123
 
-    >>> _ = engine.execute("create table asd (i integer, s varchar(50))")
-    >>> _ = engine.execute("insert into asd values (42, 'gubbe')")
-    >>> d3 = sqldict(engine, "asd", keycol="s", valcol="i")
-    >>> d4 = sqldict(engine, "asd", keycol="i", valcol="s")
+    >>> _ = e.execute("create table asd (i integer, s varchar(50))")
+    >>> _ = e.execute("insert into asd values (42, 'gubbe')")
+    >>> d3 = sqldict(e, "asd", keycol="s", valcol="i")
+    >>> d4 = sqldict(e, "asd", keycol="i", valcol="s")
     >>> assert d3["gubbe"] == 42
     >>> assert d4[42] == "gubbe"
 
+    broken (at least mysql) when combining keytype and serialize,
+    it tries to serialize 42.
+    FIXME: d[int] = Integer or d[Integer]
+    >>> from sqlalchemy import *
+    >>> d = sqldict(e, "laarge", create=1, serialize=1, keytype=Integer)
+    >>> d[42] = "galning"
     """
 
     conn = None
@@ -618,7 +633,11 @@ def sqldict(db, tablename,
                     extended=serialize,
                     create=create,
                     sort=1,
-                    **kw)
+                    keycol=keycol,
+                    valcol=valcol,
+                    keytype=keytype,
+                    valtype=valtype,
+                    )
 
 
 if __name__ == '__main__':
